@@ -18,6 +18,21 @@ sub execute {
     my ($self, $handler, $rows, $prefetch) = @_;
     return unless @$rows;
 
+    my %where;
+    if (ref $prefetch eq 'HASH') {
+        my %prefetch;
+        for my $key (keys %$prefetch) {
+            if ($key =~ /^\./) {
+                my $column = $key =~ s/^\.//r;
+                $where{$column} = $prefetch->{$key};
+            }
+            else {
+                $prefetch{$key} = $prefetch->{$key};
+            }
+        }
+        $prefetch = \%prefetch;
+    }
+
     my $relationship = $self->relationship;
     my $name         = $relationship->name;
     my $table_name   = $relationship->dest_table_name;
@@ -29,14 +44,28 @@ sub execute {
         my $src_column  = $src_columns[0];
         my $dest_column = $dest_columns[0];
 
+        my @src_values = grep defined, map { $_->get_column($src_column) } @$rows;
+        unless (@src_values) {
+            # set empty value
+            for my $row (@$rows) {
+                $row->relay_data->{$name} = $has_many ? [] : undef;
+            }
+            return;
+        }
+
         my @related_rows = $handler->select($table_name => {
-            $dest_column => sql_in([grep defined, map { $_->get_column($src_column) } @$rows])
+            %where,
+            $dest_column => sql_in(\@src_values),
         }, { prefetch => $prefetch })->all;
 
         my %related_rows_map = partition_by { $_->get_column($dest_column) } @related_rows;
         for my $row (@$rows) {
             my $src_value = $row->get_column($src_column);
-            next unless defined $src_value;
+            unless (defined $src_value) {
+                # set empty value
+                $row->relay_data->{$name} = $has_many ? [] : undef;
+                next;
+            }
 
             my $related_rows = $related_rows_map{$src_value};
             $row->relay_data->{$name} = $has_many ? $related_rows : $related_rows->[0];
@@ -49,6 +78,7 @@ sub execute {
         for my $row (@$rows) {
             next if notall { defined $row->get_column($_) } @src_columns;
             my @related_rows = $handler->select($table_name => {
+                %where,
                 pairwise { $a => $row->get_column($b) } @dest_columns, @src_columns
             }, { prefetch => $prefetch })->all;
             $row->relay_data->{$name} = $has_many ? \@related_rows : $related_rows[0];
